@@ -36,6 +36,7 @@
 #include "gupnp-didl-lite-descriptor-private.h"
 #include "gupnp-didl-lite-container.h"
 #include "gupnp-didl-lite-item.h"
+#include "gupnp-didl-lite-contributor-private.h"
 #include "xml-util.h"
 
 G_DEFINE_ABSTRACT_TYPE (GUPnPDIDLLiteObject,
@@ -46,8 +47,8 @@ struct _GUPnPDIDLLiteObjectPrivate {
         xmlNode     *xml_node;
         GUPnPXMLDoc *xml_doc;
 
-        xmlNs              *upnp_ns;
-        xmlNs              *dc_ns;
+        xmlNs *upnp_ns;
+        xmlNs *dc_ns;
 };
 
 enum {
@@ -467,6 +468,10 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
          * GUPnPDIDLLiteObject:creator
          *
          * The creator of this object.
+         *
+         * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_get_creators and
+         * #gupnp_didl_lite_object_add_creator instead since unlike this
+         * property, they are capable of dealing with multiple creator nodes.
          **/
         g_object_class_install_property
                 (object_class,
@@ -484,6 +489,10 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
          * GUPnPDIDLLiteObject:artist
          *
          * The artist of this object.
+         *
+         * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_get_artists and
+         * #gupnp_didl_lite_object_add_artist instead since unlike this
+         * property, they are capable of dealing with multiple artist nodes.
          **/
         g_object_class_install_property
                 (object_class,
@@ -501,6 +510,10 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
          * GUPnPDIDLLiteObject:author
          *
          * The author of this object.
+         *
+         * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_get_authors and
+         * #gupnp_didl_lite_object_add_author instead since unlike this
+         * property, they are capable of dealing with multiple author nodes.
          **/
         g_object_class_install_property
                 (object_class,
@@ -664,8 +677,36 @@ is_resource_compatible (GUPnPDIDLLiteResource *resource,
         return ret;
 }
 
+static GList *
+get_contributor_list_by_name (GUPnPDIDLLiteObject *object,
+                              const char          *name)
+{
+        GList *contributors = NULL;
+        GList *ret = NULL;
+        GList *l;
+
+        contributors = gupnp_didl_lite_object_get_properties (object, name);
+
+        for (l = contributors; l; l = l->next) {
+                GUPnPDIDLLiteContributor *contributor;
+                xmlNode *contributor_node;
+
+                contributor_node = (xmlNode *) l->data;
+
+                contributor = gupnp_didl_lite_contributor_new_from_xml
+                                        (contributor_node,
+                                         object->priv->xml_doc);
+
+                ret = g_list_append (ret, contributor);
+        }
+
+        g_list_free (contributors);
+
+        return ret;
+}
+
 /**
- * gupnp_didl_lite_object_new_from_xml
+ * gupnp_didl_lite_object_new_from_xml:
  * @xml_node: The pointer to 'res' node in XML document
  * @xml_doc: The reference to XML document containing this object
  *
@@ -696,7 +737,7 @@ gupnp_didl_lite_object_new_from_xml (xmlNode     *xml_node,
 }
 
 /**
- * gupnp_didl_lite_object_get_xml_node
+ * gupnp_didl_lite_object_get_xml_node:
  * @object: The #GUPnPDIDLLiteObject
  *
  * Get the pointer to object node in XML document.
@@ -712,7 +753,7 @@ gupnp_didl_lite_object_get_xml_node (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_upnp_class
+ * gupnp_didl_lite_object_get_upnp_class:
  * @object: The #GUPnPDIDLLiteObject
  *
  * Get the UPnP class of the @object.
@@ -745,7 +786,7 @@ gupnp_didl_lite_object_get_id (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_parent_id
+ * gupnp_didl_lite_object_get_parent_id:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the ID of the parent of the @object.
@@ -763,15 +804,15 @@ gupnp_didl_lite_object_get_parent_id (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_properties
+ * gupnp_didl_lite_object_get_properties:
  * @object: #GUPnPDIDLLiteObject
  * @name: name of the properties
  *
  * Use this function to retreive property nodes by name.
  *
- * Return value: The list of property nodes by the name @property_name
- * belonging to @object, or %NULL. #g_list_free the returned list after
- * usage but do not modify the contents.
+ * Return value: (element-type utf8) (transfer none): The list of property nodes
+ * by the name @property_name belonging to @object, or %NULL. #g_list_free the
+ * returned list after usage but do not modify the contents.
  **/
 GList *
 gupnp_didl_lite_object_get_properties (GUPnPDIDLLiteObject *object,
@@ -785,7 +826,7 @@ gupnp_didl_lite_object_get_properties (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_get_restricted
+ * gupnp_didl_lite_object_get_restricted:
  * @object: #GUPnPDIDLLiteObject
  *
  * Whether the @object is restricted or not.
@@ -802,7 +843,7 @@ gupnp_didl_lite_object_get_restricted (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_title
+ * gupnp_didl_lite_object_get_title:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the title of the @object.
@@ -819,12 +860,14 @@ gupnp_didl_lite_object_get_title (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_creator
+ * gupnp_didl_lite_object_get_creator:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the creator of the @object.
  *
  * Return value: The creator of the @object, or %NULL.
+ *
+ * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_get_creators instead.
  **/
 const char *
 gupnp_didl_lite_object_get_creator (GUPnPDIDLLiteObject *object)
@@ -836,14 +879,32 @@ gupnp_didl_lite_object_get_creator (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_artist
+ * gupnp_didl_lite_object_get_creators:
+ * @object: #GUPnPDIDLLiteObject
+ *
+ * Get the creators of the @object.
+ *
+ * Return value: The list of creators belonging to @object, or %NULL.
+ * #g_list_free the returned list after usage and unref each object in it.
+ **/
+GList *
+gupnp_didl_lite_object_get_creators (GUPnPDIDLLiteObject *object)
+{
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        return get_contributor_list_by_name (object, "creator");
+}
+
+/**
+ * gupnp_didl_lite_object_get_artist:
  * @object: #GUPnPDIDLLiteObject
- * @role: Location to put the role string (if any) into, or %NULL
  *
  * Get the artist of the @object. If role is not %NULL, it is set to the role
  * of the artist if available.
  *
  * Return value: The artist of the @object, or %NULL.
+ *
+ * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_get_artists instead.
  **/
 const char *
 gupnp_didl_lite_object_get_artist (GUPnPDIDLLiteObject *object)
@@ -855,14 +916,31 @@ gupnp_didl_lite_object_get_artist (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_author
+ * gupnp_didl_lite_object_get_artists:
  * @object: #GUPnPDIDLLiteObject
- * @role: Location to put the role string (if any) into, or %NULL
  *
- * Get the author of the @object. If role is not %NULL, it is set to the role
- * of the author if available.
+ * Get the artists of the @object.
+ *
+ * Return value: The list of artists belonging to @object, or %NULL.
+ * #g_list_free the returned list after usage and unref each object in it.
+ **/
+GList *
+gupnp_didl_lite_object_get_artists (GUPnPDIDLLiteObject *object)
+{
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        return get_contributor_list_by_name (object, "artists");
+}
+
+/**
+ * gupnp_didl_lite_object_get_author:
+ * @object: #GUPnPDIDLLiteObject
+ *
+ * Get the author of the @object.
  *
  * Return value: The author of the @object, or %NULL.
+ *
+ * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_get_authors instead.
  **/
 const char *
 gupnp_didl_lite_object_get_author (GUPnPDIDLLiteObject *object)
@@ -874,13 +952,65 @@ gupnp_didl_lite_object_get_author (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_genre
+ * gupnp_didl_lite_object_get_authors:
  * @object: #GUPnPDIDLLiteObject
- * @id: Location to put the ID string (if any) into, or %NULL
- * @extended: Location to put the extended genre string (if any) into, or %NULL
  *
- * Get the genre of the @object. If @id is not %NULL, it is set to the ID
- * of the genre if available. The same goes for @extended.
+ * Get the authors of the @object.
+ *
+ * Return value: The list of authors belonging to @object, or %NULL.
+ * #g_list_free the returned list after usage and unref each object in it.
+ **/
+GList *
+gupnp_didl_lite_object_get_authors (GUPnPDIDLLiteObject *object)
+{
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        return get_contributor_list_by_name (object, "authors");
+}
+
+/**
+ * gupnp_didl_lite_object_get_descriptors:
+ * @object: #GUPnPDIDLLiteObject
+ *
+ * Get the descriptors of the @object.
+ *
+ * Return value: The list of descriptors belonging to @object, or %NULL.
+ * #g_list_free the returned list after usage and unref each object in it.
+ **/
+GList *
+gupnp_didl_lite_object_get_descriptors (GUPnPDIDLLiteObject *object)
+{
+        GList *descriptors = NULL;
+        GList *ret = NULL;
+        GList *l;
+
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        descriptors = gupnp_didl_lite_object_get_properties (object, "desc");
+
+        for (l = descriptors; l; l = l->next) {
+                GUPnPDIDLLiteDescriptor *descriptor;
+                xmlNode *descriptor_node;
+
+                descriptor_node = (xmlNode *) l->data;
+
+                descriptor = gupnp_didl_lite_descriptor_new_from_xml
+                                        (descriptor_node,
+                                         object->priv->xml_doc);
+
+                ret = g_list_append (ret, descriptor);
+        }
+
+        g_list_free (descriptors);
+
+        return ret;
+}
+
+/**
+ * gupnp_didl_lite_object_get_genre:
+ * @object: #GUPnPDIDLLiteObject
+ *
+ * Get the genre of the @object.
  *
  * Return value: The genre of the @object, or %NULL.
  **/
@@ -894,7 +1024,7 @@ gupnp_didl_lite_object_get_genre (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_write_status
+ * gupnp_didl_lite_object_get_write_status:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the write status of the @object.
@@ -911,7 +1041,7 @@ gupnp_didl_lite_object_get_write_status (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_album
+ * gupnp_didl_lite_object_get_album:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the album of the @object.
@@ -928,7 +1058,7 @@ gupnp_didl_lite_object_get_album (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_album_art
+ * gupnp_didl_lite_object_get_album_art:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the URI to album art of the @object.
@@ -945,7 +1075,7 @@ gupnp_didl_lite_object_get_album_art (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_description
+ * gupnp_didl_lite_object_get_description:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the description of the @object.
@@ -962,7 +1092,7 @@ gupnp_didl_lite_object_get_description (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_date
+ * gupnp_didl_lite_object_get_date:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the date of the @object.
@@ -979,7 +1109,7 @@ gupnp_didl_lite_object_get_date (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_track_number
+ * gupnp_didl_lite_object_get_track_number:
  * @object: #GUPnPDIDLLiteObject
  *
  * Get the original track number of the @object.
@@ -1002,12 +1132,13 @@ gupnp_didl_lite_object_get_track_number (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_resources
+ * gupnp_didl_lite_object_get_resources:
  * @object: #GUPnPDIDLLiteObject
  *
  * Use this function to retreive resources from the @object.
  *
- * Return value: The list of resources belonging to @object, or %NULL.
+ * Return value: (element-type GUPnPDIDLLiteResource) (transfer full): The list of resources belonging to 
+@object, or %NULL.
  * #g_list_free the returned list after usage and unref each resource in it.
  **/
 GList *
@@ -1041,7 +1172,7 @@ gupnp_didl_lite_object_get_resources (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_get_compat_resource
+ * gupnp_didl_lite_object_get_compat_resource:
  * @object: #GUPnPDIDLLiteObject
  * @sink_protocol_info: The SinkProtocolInfo string from MediaRenderer
  * @lenient: Enable lenient mode
@@ -1073,6 +1204,8 @@ gupnp_didl_lite_object_get_compat_resource
         g_return_val_if_fail (sink_protocol_info != NULL, NULL);
 
         resources = gupnp_didl_lite_object_get_resources (object);
+        if (resources == NULL)
+                return NULL;
 
         for (res = resources;
              res != NULL && resource == NULL;
@@ -1097,8 +1230,9 @@ gupnp_didl_lite_object_get_compat_resource
 }
 
 /**
- * gupnp_didl_lite_object_set_upnp_class
+ * gupnp_didl_lite_object_set_upnp_class:
  * @object: The #GUPnPDIDLLiteObject
+ * @upnp_class: The UPnP class as string.
  *
  * Set the UPnP class of the @object to @upnp_class.
  **/
@@ -1140,7 +1274,7 @@ gupnp_didl_lite_object_set_id (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_parent_id
+ * gupnp_didl_lite_object_set_parent_id:
  * @object: #GUPnPDIDLLiteObject
  * @parent_id: The parent ID
  *
@@ -1161,7 +1295,7 @@ gupnp_didl_lite_object_set_parent_id (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_restricted
+ * gupnp_didl_lite_object_set_restricted:
  * @object: #GUPnPDIDLLiteObject
  * @restricted: The restricted status
  *
@@ -1188,7 +1322,7 @@ gupnp_didl_lite_object_set_restricted (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_title
+ * gupnp_didl_lite_object_set_title:
  * @object: #GUPnPDIDLLiteObject
  * @title: The title
  *
@@ -1211,11 +1345,13 @@ gupnp_didl_lite_object_set_title (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_creator
+ * gupnp_didl_lite_object_set_creator:
  * @object: #GUPnPDIDLLiteObject
  * @creator: The creator
  *
  * Set the creator of the @object to @creator.
+ *
+ * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_add_creator instead.
  **/
 void
 gupnp_didl_lite_object_set_creator (GUPnPDIDLLiteObject *object,
@@ -1234,11 +1370,40 @@ gupnp_didl_lite_object_set_creator (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_artist
+ * gupnp_didl_lite_object_add_creator:
+ * @object: The #GUPnPDIDLLiteObject
+ *
+ * Add a new creator node to the @object and return the associated
+ * #GUPnPDIDLLiteContributor object.
+ *
+ * Return value: A new #GUPnPDIDLLiteContributor object. Unref after usage.
+ **/
+GUPnPDIDLLiteContributor *
+gupnp_didl_lite_object_add_creator (GUPnPDIDLLiteObject *object)
+{
+        xmlNode *res_node;
+
+        g_return_val_if_fail (object != NULL, NULL);
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        res_node = xmlNewChild (object->priv->xml_node,
+                                object->priv->dc_ns,
+                                (unsigned char *) "creator",
+                                NULL);
+
+        return gupnp_didl_lite_contributor_new_from_xml (res_node,
+                                                         object->priv->xml_doc);
+}
+
+
+/**
+ * gupnp_didl_lite_object_set_artist:
  * @object: The #GUPnPDIDLLiteObject
  * @artist: The Artist
  *
  * Set the Artist of the @object to @artist.
+ *
+ * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_add_artist instead.
  **/
 void
 gupnp_didl_lite_object_set_artist (GUPnPDIDLLiteObject *object,
@@ -1257,11 +1422,39 @@ gupnp_didl_lite_object_set_artist (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_author
+ * gupnp_didl_lite_object_add_artist:
+ * @object: The #GUPnPDIDLLiteObject
+ *
+ * Add a new Artist node to the @object and return the associated
+ * #GUPnPDIDLLiteContributor object.
+ *
+ * Return value: A new #GUPnPDIDLLiteContributor object. Unref after usage.
+ **/
+GUPnPDIDLLiteContributor *
+gupnp_didl_lite_object_add_artist (GUPnPDIDLLiteObject *object)
+{
+        xmlNode *res_node;
+
+        g_return_val_if_fail (object != NULL, NULL);
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        res_node = xmlNewChild (object->priv->xml_node,
+                                object->priv->upnp_ns,
+                                (unsigned char *) "artist",
+                                NULL);
+
+        return gupnp_didl_lite_contributor_new_from_xml (res_node,
+                                                         object->priv->xml_doc);
+}
+
+/**
+ * gupnp_didl_lite_object_set_author:
  * @object: The #GUPnPDIDLLiteObject
  * @author: The Author
  *
  * Set the Author of the @object to @author.
+ *
+ * Deprecated: 0.5.3: Use #gupnp_didl_lite_object_add_author instead.
  **/
 void
 gupnp_didl_lite_object_set_author (GUPnPDIDLLiteObject *object,
@@ -1280,7 +1473,33 @@ gupnp_didl_lite_object_set_author (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_genre
+ * gupnp_didl_lite_object_add_author:
+ * @object: The #GUPnPDIDLLiteObject
+ *
+ * Add a new author node to the @object and return the associated
+ * #GUPnPDIDLLiteContributor object.
+ *
+ * Return value: A new #GUPnPDIDLLiteContributor object. Unref after usage.
+ **/
+GUPnPDIDLLiteContributor *
+gupnp_didl_lite_object_add_author (GUPnPDIDLLiteObject *object)
+{
+        xmlNode *res_node;
+
+        g_return_val_if_fail (object != NULL, NULL);
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        res_node = xmlNewChild (object->priv->xml_node,
+                                object->priv->upnp_ns,
+                                (unsigned char *) "author",
+                                NULL);
+
+        return gupnp_didl_lite_contributor_new_from_xml (res_node,
+                                                         object->priv->xml_doc);
+}
+
+/**
+ * gupnp_didl_lite_object_set_genre:
  * @object: The #GUPnPDIDLLiteObject
  * @genre: The Genre
  *
@@ -1303,7 +1522,7 @@ gupnp_didl_lite_object_set_genre (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_write_status
+ * gupnp_didl_lite_object_set_write_status:
  * @object: #GUPnPDIDLLiteObject
  * @write_status: The write status string
  *
@@ -1326,7 +1545,7 @@ gupnp_didl_lite_object_set_write_status (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_album
+ * gupnp_didl_lite_object_set_album:
  * @object: #GUPnPDIDLLiteObject
  * @album: The album string
  *
@@ -1349,7 +1568,7 @@ gupnp_didl_lite_object_set_album (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_album_art
+ * gupnp_didl_lite_object_set_album_art:
  * @object: #GUPnPDIDLLiteObject
  * @album_art: The URI of album art
  *
@@ -1372,7 +1591,7 @@ gupnp_didl_lite_object_set_album_art (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_description
+ * gupnp_didl_lite_object_set_description:
  * @object: #GUPnPDIDLLiteObject
  * @description: The description string
  *
@@ -1395,7 +1614,7 @@ gupnp_didl_lite_object_set_description (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_date
+ * gupnp_didl_lite_object_set_date:
  * @object: #GUPnPDIDLLiteObject
  * @date: The date string
  *
@@ -1418,7 +1637,7 @@ gupnp_didl_lite_object_set_date (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_set_track_number
+ * gupnp_didl_lite_object_set_track_number:
  * @object: #GUPnPDIDLLiteObject
  * @track_number: The original track number
  *
@@ -1445,7 +1664,7 @@ gupnp_didl_lite_object_set_track_number (GUPnPDIDLLiteObject *object,
 }
 
 /**
- * gupnp_didl_lite_object_add_resource
+ * gupnp_didl_lite_object_add_resource:
  * @object: A #GUPnPDIDLLiteObject
  *
  * Creates a new resource, attaches it to @object and returns it.
@@ -1469,7 +1688,7 @@ gupnp_didl_lite_object_add_resource (GUPnPDIDLLiteObject *object)
 }
 
 /**
- * gupnp_didl_lite_object_add_descriptor
+ * gupnp_didl_lite_object_add_descriptor:
  * @object: A #GUPnPDIDLLiteObject
  *
  * Creates a new descriptor, attaches it to @object and returns it.
